// __multiversion__
// This signals the loading code to prepend either #version 100 or #version 300 es as apropriate.

#include "fragmentVersionCentroidUV.h"

#include "uniformEntityConstants.h"
#include "uniformShaderConstants.h"
#include "uniformPerFrameConstants.h"

// Including files...
#include "gl_assets/functionLib.glsl"
#include "gl_assets/globalPrecision.glsl"

LAYOUT_BINDING(0) uniform sampler2D TEXTURE_0;
LAYOUT_BINDING(1) uniform sampler2D TEXTURE_1;

#ifdef EFFECTS_OFFSET
#include "uniformEffectsConstants.h"
#endif

varying vec4 color;

#ifdef ENABLE_FOG
varying vec4 fogColor;
#endif

#ifdef GLINT
	varying vec2 layer1UV;
	varying vec2 layer2UV;

	vec4 glintBlend(vec4 dest, vec4 source) {
		return vec4(source.rgb * source.rgb, 0.0) + dest;
	}
#endif

void main()
{
#ifdef EFFECTS_OFFSET
	vec4 diffuse = texture2D( TEXTURE_0,  uv + EFFECT_UV_OFFSET);
#else
	vec4 diffuse = texture2D( TEXTURE_0,  uv );
#endif

#ifdef MULTI_COLOR_TINT
	// Texture is a mask for tinting with two colors
	vec2 colorMask = diffuse.rg;

	// Apply the base color tint
	diffuse.rgb = colorMask.rrr * color.rgb;

	// Apply the secondary color mask and tint so long as its grayscale value is not 0
	diffuse.rgb = mix(diffuse, colorMask.gggg * CHANGE_COLOR, ceil(colorMask.g)).rgb;
#endif

#ifdef ALPHA_TEST
#ifdef ENABLE_VERTEX_TINT_MASK
	if(diffuse.a <= 0.0)
#else
	if(diffuse.a < 0.5)
#endif
	 	discard;
#endif

#if defined(ENABLE_VERTEX_TINT_MASK) && !defined(MULTI_COLOR_TINT)
	diffuse.rgb = mix(diffuse.rgb, diffuse.rgb*color.rgb, diffuse.a);
	if (color.a > 0.0) {
		diffuse.a = (diffuse.a > 0.0) ? 1.0 : 0.0; // This line is needed for horse armour icon and dyed leather to work properly
	}
#endif

#ifdef GLINT
	vec4 layer1 = texture2D(TEXTURE_1, fract(layer1UV)).rgbr * GLINT_COLOR;
	vec4 layer2 = texture2D(TEXTURE_1, fract(layer2UV)).rgbr * GLINT_COLOR;
	vec4 glint = (layer1 + layer2);
	glint.rgb *= color.a;

	#ifdef INVENTORY
		diffuse.rgb = glint.rgb;
	#else
		diffuse.rgb = glintBlend(diffuse, glint).rgb;
	#endif
#endif

#ifdef USE_OVERLAY
	//use either the diffuse or the OVERLAY_COLOR
	diffuse.rgb = mix(diffuse, OVERLAY_COLOR, OVERLAY_COLOR.a).rgb;
#endif

#ifdef ENABLE_VERTEX_TINT_MASK

#ifdef ENABLE_CURRENT_ALPHA_MULTIPLY
	diffuse = diffuse * vec4(1.0, 1.0, 1.0, HUD_OPACITY);
#endif

#elif !defined(MULTI_COLOR_TINT)
	diffuse = diffuse * color;
#endif

// Main variables....
dayA = pow(clamp2(1. - FOG_COLOR.b * 1.2), .5);
nightA = pow(clamp2(1. - FOG_COLOR.r * 1.5),1.2);
rainA = clamp2(pow((.7 - FOG_CONTROL.x) / (FOG_CONTROL.y - FOG_CONTROL.x), 3.));

vec3 shadowAmbient = shadowCol * shadow_B * diffuse.rgb;
float uv1x = maxC(TILE_LIGHT_COLOR);
float uv2x = clamp2(pow(uv1x, lightShrp) * lightSize);
float shadow_v = shdAlpha * 1.2 * (1. - uv2x) * mix(1., .64, max(nightA, rainA));

#if defined USE_LIGHTING && defined BASIC_SHADOWS
	diffuse.rgb = diffuse.rgb * (1. - shadow_v) + shadowAmbient * shadow_v;
#else
	diffuse.rgb *= emissValue;
#endif

#if defined UNDERWATER && defined UNDERWATER_CAUSTIC
	diffuse.rgb *= FOG_COLOR.rgb * (1. - uv2x) + uv2x;
#endif

diffuse.rgb = toneA(diffuse.rgb);
vec3 grey_s = A_Saturation(diffuse.rgb, monoSatE * (1. - uv2x) + uv2x);

#ifndef UNDERWATER // The monochromatic effect will render differently for terrain, if the player is not underwater
	diffuse.rgb = FOG_CONTROL.x < 1. && FOG_CONTROL.x > 0. ? diffuse.rgb * (1. - rainA) + grey_s.rgb * mix(mono_B, 1., uv2x) * rainA : diffuse.rgb;
#endif

#ifdef HDR
	diffuse = rgb2hdr(diffuse);
#endif

	// Fog needs to be applied after the color tinting.
#ifdef ENABLE_FOG
	//apply fog
	diffuse.rgb = mix(diffuse.rgb, fogColor.rgb, fogColor.a);
#endif

	gl_FragColor = diffuse;
}
